#!/bin/bash

if [[ ! -w /etc/passwd ]] ; then
  echo "Only 'root' can execute this script."
  exit 1
fi

#
# Set epages environmental variables
#
if [[ -f /etc/sysconfig/epages6 ]] ; then
  sed -i 's/\(EPAGES_MYSQL_CONNECTION\)=\([a-z].*\)/\1="\2"/i' /etc/sysconfig/epages6
  . /etc/sysconfig/epages6
fi
EPAGES_PROJECT=${EPAGES_PROJECT:-$EPAGES_INSTALL_RUN_SCRIPT}
EPAGES_PROJECT=${EPAGES_PROJECT:-epages6}

die() {
  touch $EPAGES/Setup/mysql.ini
  exit 0
}

void_var() {
  echo "Following variable value(s) may be incorrect:"
  for i in $@ ; do echo "  $i=\"$(eval echo '$'$i)\"" ; done
  echo "Check these variable(s) in /etc/sysconfig/epages6 and rerun:"
  echo "  $0"
  die
}

ls /etc/default/$EPAGES_PROJECT >/dev/null || die

sed -i 's/\(EPAGES_MYSQL_CONNECTION\)=\([a-z].*\)/\1="\2"/i' /etc/default/$EPAGES_PROJECT
. /etc/default/$EPAGES_PROJECT

#
# Read the connection settings from sysconfig file
#
MYSQL_SOCKET=${EPAGES_INSTALL_MYSQL_CONNECTION:-/var/lib/mysql/mysql.sock}

INI_CONFIG="$EPAGES"/bin/ini-config.sh
"$INI_CONFIG" -f /etc/default/$EPAGES_PROJECT -k EPAGES_MYSQL_CONNECTION -v "\"$MYSQL_SOCKET\"" -i

#
# Set the mysql connection string based on the decision if we have a socket or TCP connection here.
#
if [[ ${MYSQL_SOCKET#/} != $MYSQL_SOCKET ]] ; then
  ls -d $MYSQL_SOCKET >/dev/null || void_var EPAGES_INSTALL_MYSQL_CONNECTION
  SOCKET_CONNECTION=1
  MYSQL_CONNECTION_ARG=-S$MYSQL_SOCKET
  export MYSQL_CONN="mysql_socket=$MYSQL_SOCKET"
else
  MYSQL_HOST=${MYSQL_SOCKET%;*}
  [[ $MYSQL_HOST = $MYSQL_SOCKET ]] && MYSQL_PORT=3306 || {
    MYSQL_PORT=${MYSQL_SOCKET##*;}
  }
  MYSQL_CONNECTION_ARG="-h $MYSQL_HOST -P $MYSQL_PORT"
  unset MYSQL_SOCKET
  export MYSQL_CONN="hostname=$MYSQL_HOST;port=$MYSQL_PORT"
fi

#
# Read the backup user credentials.
#
unset MYSQL_PASS_ARG
export MYSQL_USER=${EPAGES_INSTALL_MYSQL_USER:-root}
export MYSQL_PASS=$EPAGES_INSTALL_MYSQL_PASS
MYSQL_PASS_ARG=${MYSQL_PASS:+"-p'$MYSQL_PASS'"}

#
# Test the connection based on the connection string.
#
echo "mysql $MYSQL_CONNECTION_ARG -u$MYSQL_USER $MYSQL_PASS_ARG" >/tmp/proc.~$$~
echo exit | eval mysql $MYSQL_CONNECTION_ARG -u$MYSQL_USER $MYSQL_PASS_ARG 2>>/tmp/proc.~$$~ || {
  echo exit | eval mysql -u$MYSQL_USER $MYSQL_PASS_ARG 2>/dev/null && MYSQL_CONNECTION_ARG=
}
RET=$?
if [[ $RET -ne 0 ]] ; then
  if [[ -z $MYSQL_PASS ]] ; then
    MYSQL_PASS=epages
    MYSQL_PASS_ARG=-p$MYSQL_PASS
    echo exit | eval mysql $MYSQL_CONNECTION_ARG -u$MYSQL_USER $MYSQL_PASS_ARG 2>/dev/null || {
      echo exit | eval mysql -u$MYSQL_USER $MYSQL_PASS_ARG 2>/dev/null && MYSQL_CONNECTION_ARG=
    }
    RET=$?
  fi
  if [[ $RET -ne 0 ]] ; then
    cat /tmp/proc.~$$~
    rm -f /tmp/proc.~$$~
    void_var EPAGES_INSTALL_MYSQL_CONNECTION EPAGES_INSTALL_MYSQL_USER EPAGES_INSTALL_MYSQL_PASS
  fi
fi
rm -f /tmp/proc.~$$~

#
# grant all privileges on *.* to $MYSQL_USER@"$EPAGES_HOSTS"
#
[[ -z $MYSQL_PASS ]] && IDENTIFIED_BY= || IDENTIFIED_BY="IDENTIFIED BY '$MYSQL_PASS'"
for i in $EPAGES_INSTALL_AS_HOST $EPAGES_INSTALL_DB_HOST $MYSQL_HOST ; do
  echo 'GRANT ALL ON *.* TO "$MYSQL_USER"@"$i" $IDENTIFIED_BY WITH GRANT OPTION;' \
  | eval mysql $MYSQL_SOCKET_ARG -u$MYSQL_USER $MYSQL_PASS_ARG >/dev/null 2>&1
done

#
# Rewrite the database .conf files to update the datasource string and the backup user credentials.
#
$EPAGES_PERL/bin/perl -MConfig::IniFiles -MFile::Copy -e '
use DE_EPAGES::Database::API::Connection qw(EncryptDBPassword);
my $EncPasswd;
$EncPasswd = EncryptDBPassword($ENV{MYSQL_PASS}) if $ENV{MYSQL_PASS};
foreach my $Conn (qw(Site Store Backup)) {
  my $DBC = "$ENV{EPAGES_CONFIG}/Database.d/$Conn.conf";
  copy("$DBC.initial", $DBC) unless -f $DBC;
  tie %Conf, "Config::IniFiles", (-file => $DBC) or die "$DBC: $!\n";
  $Conf{Connection}{datasource} = "dbi:mysql:$ENV{MYSQL_CONN}";
  if ($Conn eq "Backup") {
    $Conf{Connection}{login} = $ENV{MYSQL_USER};
    $Conf{Connection}{encpasswd} = $EncPasswd if $ENV{MYSQL_PASS};
    delete $Conf{Connection}{passwd};
  }
  tied( %Conf )->RewriteConfig() or die "$DBC: $!\n";
  chown((getpwnam("ep_appl"))[2,3], $DBC);
}
' || die

rm -f $EPAGES/Setup/mysql.ini
